#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import sys
import time
import json
from resources.lib.api import API, TokenNotFound, TokenRenewFailed
from resources.lib.common import build_url, getInputstreamAddon
from resources.lib.database import Database

import xbmc
import xbmcvfs
import xbmcgui
import xbmcplugin
import xbmcaddon
import inputstreamhelper
import base64
import re

addon_handle = int(sys.argv[1])

class Navigation():
    def _addContextMenuFav(self, itemID, listItem):
        if itemID in self._favs:
            label = "Aus Watchlist entfernen"
            action = "removeFav"
        else:
            label = "Zu Watchlist hinzufügen"
            action = "addFav"
        commands = []
        url = f'RunPlugin({sys.argv[0]}?action={action}&id={itemID})'
        commands.append(( label, url))
        listItem.addContextMenuItems( commands )

    @staticmethod
    def _getInfoLabel(data): # EDITED = 26.06.2024 @realvito
        info = {}
        info['title'] = data.title
        info['year'] = data.productionYear
        info['plot'] = data.description if data.description not in ['', None] else ' '
        info['country'] = data.productionCountries
        if data.genres is not None and len(data.genres) > 0:
            info['genre'] = data.genres[0]
        info['duration'] = data.duration
        info['episode'] = data.episodeNo
        if data.season == None:
            info['season'] = 1
        else:
            info['season'] = data.season
        return info

    def _setLoginPW(self):
        keyboard = xbmc.Keyboard('', 'Passwort', True)
        keyboard.doModal(60000)
        if (keyboard.isConfirmed() and keyboard.getText() and
            len(keyboard.getText()) >= 6):
            password = keyboard.getText()
            return password
        return ''

    def __init__(self, ):
        self._addon = xbmcaddon.Addon()
        self._db = None
        self.datapath = xbmcvfs.translatePath(self._addon.getAddonInfo('profile'))
        dbPath = self.datapath + 'Database'
        if not os.path.isdir(self.datapath):
            os.mkdir(self.datapath)
        self._db = Database(dbPath)
        self._showPremium = False
        self._showlive = False
        self._showLivePay = False
        premiereEnabled = self._addon.getSetting('premiere_enabled') == "true"
        self.user = self._addon.getSetting('email')
        token = self._addon.getSetting('acc_token')
        refresh_token = self._addon.getSetting('refresh_token')
        lastUpdate = self._addon.getSetting('versionLastUpdate')
        lastUpdateInt = 0
        try:
            if lastUpdate != "":
                lastUpdateInt = float(lastUpdate)
        except:
            pass
        if  lastUpdateInt > time.time() - 60*60*24 :
            clientID = self._addon.getSetting('clientID')
            version = self._addon.getSetting('version')
        else:
            clientID = API.getClientID()
            version = API.getClientVersion()
            self._addon.setSetting('clientID', clientID)
            self._addon.setSetting('version', version)
            self._addon.setSetting('versionLastUpdate', str(time.time()))
        try:
            self.api = API(token, refresh_token, clientID=clientID, clientVersion=version, fetchPremiere=premiereEnabled)
        except TokenNotFound:
            xbmcgui.Dialog().notification('Token nicht gefunden', 'Ohne Token funktioniert das Addon nicht.', icon=xbmcgui.NOTIFICATION_ERROR)
            return
        except TokenRenewFailed:
            xbmcgui.Dialog().notification('Login fehlgeschlagen', 'Bitte erneut anmelden', icon=xbmcgui.NOTIFICATION_ERROR)
            self.api = API(clientID=clientID, clientVersion=version, fetchPremiere=premiereEnabled)
            self.login()

        if token != self.api.getToken():
            self._addon.setSetting('acc_token', self.api.getToken())
        if refresh_token != self.api.getRefreshToken():
            self._addon.setSetting('refresh_token', self.api.getRefreshToken())
        self._licence_url = ("{licenseURL}|"
            + "User-Agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:122.0) Gecko/20100101 Firefox/122.0"
            + "&x-auth-token={TOKEN}&content-type=text/html&Referer="+base64.b64decode(b'aHR0cHM6Ly9wbHVzLnJ0bC5kZS8=').decode()+"|R{{SSM}}|")
        self._helperActivated = self._addon.getSetting(
            'is_helper_enabled') == "true"
        self._recFile = xbmcvfs.translatePath(self._addon.getAddonInfo('profile')) + "recomendations.json"
        try:
            base64Parts = token.split(".")
            userToken = "%s==" % base64Parts[1]
            userData = json.loads(base64.b64decode(userToken))
            if "permissions" in userData:
                if "streaming" in userData["permissions"]:
                    if "vodAccessToPayContent" in userData["permissions"]["streaming"] and userData["permissions"]["streaming"]["vodAccessToPayContent"]==True:
                        self._showPremium = True
                    if "livestreamAccessToPayTv" in userData["permissions"]["streaming"] and userData["permissions"]["streaming"]["livestreamAccessToPayTv"]==True:
                        self._showLivePay = True
                    if "livestreamAccessToFreeTv" in userData["permissions"]["streaming"] and userData["permissions"]["streaming"]["livestreamAccessToFreeTv"]==True:
                        self._showlive = True
        except:
            pass
        self._favs = []
        if self.user != "":
            favs = self._db.get('favorites')
            if favs == None:
                favs = self.api.getFavs()
                self._db.set('favorites', favs)
            self._favs = favs
        #xbmcplugin.addSortMethod(addon_handle, xbmcplugin.SORT_METHOD_LABEL)

    def login(self):
        keyboard = xbmc.Keyboard('', 'E-Mail-Adresse')
        keyboard.doModal()
        if keyboard.isConfirmed() and keyboard.getText():
            username = keyboard.getText()
            login = self.api.Login(self.api)
            password = self._setLoginPW()
            if password != '':
                if login.sendLogin(username, password):
                    accessToken = login.getAccessToken()
                    self._addon.setSetting('acc_token', accessToken)
                    self._addon.setSetting('refresh_token', login.getRefreshToken())
                    self._addon.setSetting('email', username)
                    xbmcgui.Dialog().notification(
                        'Login erfolgreich', 'Angemeldet als "'
                        + username + '".', icon=xbmcgui.NOTIFICATION_INFO)
                    return True
                else:
                    return False

    def search(self):
        keyboard = xbmc.Keyboard('', 'Suchbegriff')
        keyboard.doModal()
        if keyboard.isConfirmed() and keyboard.getText():
            query = keyboard.getText()
            if query != '':
                self.listSearchResult(query)

    def listLiveTV(self):
        xbmcplugin.setPluginCategory(addon_handle, "LiveTV")
        if self._showPremium:
            for item in self.api.getLive():
                url = build_url({'action': 'playVod', 'id': item.id})
                li = xbmcgui.ListItem(item.title)
                info = {}
                info['title'] = re.sub('(\[B\]|\[/B\])', '', item.title)
                info['plot'] = item.description if item.description not in ['', None] else ' '
                li.setInfo('video', info)
                li.setProperty('IsPlayable', 'true')
                artDict = {}
                artDict['fanart'] = item.landscapeImg
                artDict['poster'] = item.portraitImage
                artDict['icon'] = item.portraitImage
                li.setArt(artDict)
                xbmcplugin.addDirectoryItem(
                    handle=addon_handle, url=url, listitem=li, isFolder=False)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=False)

    def listEvents(self):
        xbmcplugin.setPluginCategory(addon_handle, "Events")
        if self._showPremium:
            for item in self.api.getEvents():
                url = build_url({'action': 'playVod', 'id': item.id})
                import datetime
                dateTime = datetime.datetime.fromtimestamp(item.startTimestamp)
                title = f"[{dateTime.strftime('%d.%m.%Y %H:%M')}] {item.title}"
                info = {}
                info["title"] = title
                info["plot"] = item.description if item.description not in ['', None] else ' '
                li = xbmcgui.ListItem(title)
                li.setInfo('video', info)
                li.setProperty('IsPlayable', 'true')
                artDict = {}
                artDict['fanart'] = item.landscapeImg
                artDict['poster'] = item.portraitImage
                artDict['icon'] = item.portraitImage
                artDict['thumb'] = item.landscapeImg
                li.setArt(artDict)
                xbmcplugin.addDirectoryItem(
                    handle=addon_handle, url=url, listitem=li, isFolder=False)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=False)

    def rootDir(self):
        url = build_url({'action': 'overview', 'id': 'SHOW'})
        li = xbmcgui.ListItem()
        li.setLabel('Shows')
        xbmcplugin.addDirectoryItem(
            handle=addon_handle, url=url, listitem=li, isFolder=True)

        url = build_url({'action': 'overview', 'id': 'SERIES'})
        li = xbmcgui.ListItem()
        li.setLabel('Serien')
        xbmcplugin.addDirectoryItem(
            handle=addon_handle, url=url, listitem=li, isFolder=True)

        url = build_url({'action': 'overview', 'id': 'MOVIE'})
        li = xbmcgui.ListItem()
        li.setLabel('Filme')
        xbmcplugin.addDirectoryItem(
            handle=addon_handle, url=url, listitem=li, isFolder=True)

        url = build_url({'action': 'listLive'})
        li = xbmcgui.ListItem()
        li.setLabel('LiveTV')
        xbmcplugin.addDirectoryItem(
            handle=addon_handle, url=url, listitem=li, isFolder=True)

        url = build_url({'action': 'listEvents'})
        li = xbmcgui.ListItem()
        li.setLabel('Events')
        xbmcplugin.addDirectoryItem(
            handle=addon_handle, url=url, listitem=li, isFolder=True)

        url = build_url({'action': 'listFavs'})
        li = xbmcgui.ListItem()
        li.setLabel('Watchlist')
        xbmcplugin.addDirectoryItem(
            handle=addon_handle, url=url, listitem=li, isFolder=True)

        url = build_url({'action': 'recommendation'})
        li = xbmcgui.ListItem()
        li.setLabel('Empfehlungen')
        xbmcplugin.addDirectoryItem(
            handle=addon_handle, url=url, listitem=li, isFolder=True)

        url = build_url({'action': 'search'})
        li = xbmcgui.ListItem()
        li.setLabel('Suche')
        xbmcplugin.addDirectoryItem(
            handle=addon_handle, url=url, listitem=li, isFolder=True)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=False)

    def listEpisodesFromSeason(self, season_id):
        episodes = self.api.getSeason(season_id)
        xbmcplugin.setContent(addon_handle, 'episodes')
        #xbmcplugin.setPluginCategory(
        #   addon_handle, Navigation.buildDirectoryName(data))
        xbmcplugin.addSortMethod(addon_handle, xbmcplugin.SORT_METHOD_EPISODE)

        for ep in episodes:
            if self._showPremium or ep.tier == None or ep.tier == "FREE":
                li = xbmcgui.ListItem()
                li.setProperty('IsPlayable', 'true')
                info = self._getInfoLabel(ep)
                info['mediatype'] = "episode"
                if ep.airDate != None:
                    import datetime
                    dateTime = datetime.datetime.fromtimestamp(ep.airDate)
                    info['aired'] = dateTime.strftime('%d.%m.%Y %H:%M')
                epName = ep.title
                li.setInfo('video', info)
                li.setLabel(epName)
                artDict = {}
                artDict['fanart'] = ep.landscapeImg
                artDict['poster'] = ep.portraitImage
                artDict['icon'] = ep.portraitImage
                artDict['thumb'] = ep.landscapeImg
                li.setArt(artDict)
                url = build_url({'action': 'playVod', 'id': ep.id})
                xbmcplugin.addDirectoryItem(
                        handle=addon_handle, url=url, listitem=li,
                        isFolder=False)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=True)

    def listSearchResult(self, query):
        formats = self.api.search(query)
        for item in formats:
            action = "showMovie" if item.formatType == "Movie" else "listPage"
            url = build_url({'action': action, 'id': item.id})
            li = xbmcgui.ListItem(item.title)
            artDict = {}
            artDict['fanart'] = item.landscapeImg
            artDict['poster'] = item.portraitImage
            artDict['icon'] = item.portraitImage
            artDict['thumb'] = item.landscapeImg
            li.setArt(artDict)
            xbmcplugin.addDirectoryItem(
                handle=addon_handle, url=url, listitem=li, isFolder=True)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=True)

    def showMovie(self, movie_id):
        ep = self.api.getMovie(movie_id)
        xbmcplugin.setContent(addon_handle, 'movies')
        if self._showPremium or ep.tier == None or ep.tier == "FREE":
            li = xbmcgui.ListItem()
            li.setProperty('IsPlayable', 'true')
            info = self._getInfoLabel(ep)
            info['mediatype'] = "movie"
            epName = ep.title
            li.setInfo('video', info)
            li.setLabel(epName)
            artDict = {}
            artDict['fanart'] = ep.landscapeImg
            artDict['poster'] = ep.portraitImage
            artDict['icon'] = ep.portraitImage
            artDict['thumb'] = ep.portraitImage
            li.setArt(artDict)
            url = build_url({'action': 'playVod', 'id': ep.id})
            xbmcplugin.addDirectoryItem(
                    handle=addon_handle, url=url, listitem=li,
                    isFolder=False)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=True)

    def listSeasonsFromSeries(self, formatID):
        format = self.api.getFormat(formatID)
        xbmcplugin.setPluginCategory(addon_handle, format.title)
        posterURL = format.portraitImage
        fanartURL = format.landscapeImg
        xbmcplugin.setContent(addon_handle, 'seasons')
        xbmcplugin.addSortMethod(addon_handle, xbmcplugin.SORT_METHOD_NONE)
        for season in format.seasons:
            url = build_url({'action': 'listSeason', 'season_id': season.id})
            li = xbmcgui.ListItem(label=season.title)
            li.setProperty('IsPlayable', 'false')
            info = {}
            info['title'] = season.title
            info['plot'] = ' '
            info['year'] = season.year
            if season.year != None and season.month != None:
                info['premiered'] = f'{season.year:04d}-{season.month:02d}-01'
                info['dateadded'] = f'{info["premiered"]} 00:00:00'
            li.setInfo('video', info)
            li.setArt({'poster': posterURL, 'icon': posterURL,
                        'fanart': fanartURL, 'thumb': fanartURL})
            xbmcplugin.addDirectoryItem(handle=addon_handle, url=url,
                                        listitem=li, isFolder=True)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=True)

    def listOverview(self, dicttype):
        xbmcplugin.addSortMethod(addon_handle, xbmcplugin.SORT_METHOD_LABEL)
        formats = self._db.get(dicttype)
        if formats == None:
            print("Updating database")
            formats = self.api.getOverview(dicttype)
            self._db.set(dicttype, formats)

        action = "showMovie" if dicttype == "MOVIE" else "listPage"
        for item in formats:
            url = build_url({'action': action, 'id': item.id})
            li = xbmcgui.ListItem(item.title)
            artDict = {}
            artDict['fanart'] = item.landscapeImg
            artDict['poster'] = item.portraitImage
            artDict['icon'] = item.portraitImage
            artDict['thumb'] = item.landscapeImg
            li.setArt(artDict)
            self._addContextMenuFav(item.id, li)
            xbmcplugin.addDirectoryItem(
                handle=addon_handle, url=url, listitem=li, isFolder=True)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=True)

    def playVOD(self, id):
        hdEnabled = self._addon.getSetting('hd_enabled') == "true"
        playBackUrl, licenseURL = self.api.getPlaybackURL(id, hdEnabled)
        drmProtected = licenseURL != ""
        patchManifest = False
        if playBackUrl != "":
            li = xbmcgui.ListItem()
            protocol = 'mpd'
            drm = 'com.widevine.alpha'
            # Inputstream settings
            if self._helperActivated and drmProtected:
                is_helper = inputstreamhelper.Helper(protocol, drm=drm)
                if is_helper.check_inputstream():
                    is_addon = is_helper.inputstream_addon
            else:
                is_addon = getInputstreamAddon()
                if not is_addon:
                    xbmcgui.Dialog().notification(
                        'TvNow Fehler', 'Inputstream Addon fehlt!',
                         xbmcgui.NOTIFICATION_ERROR, 2000, True)
                    return
            if drmProtected:
                li.setProperty(is_addon + '.license_type', drm)
                if patchManifest:
                    r = self.api.getRequest(playBackUrl, client=False)
                    data = r.text
                    data = re.sub(
                        (r'<ContentProtection[^>]*>\s*<(cenc:)?pssh[^>]*>'
                        + r'[^>]*</(cenc:)?pssh>\s*</ContentProtection>'), '', data)
                    basePath = "/".join(playBackUrl.split('/')[:-1])
                    data = re.sub(
                        r'<BaseURL>([^>]*)</BaseURL>',
                        '<BaseURL>'+basePath+r'/\1</BaseURL>', data)
                    mpdFile = self.datapath + "temp.mpd"
                    try:
                        os.remove(mpdFile)
                    except OSError:
                        pass
                    with open(mpdFile, "w", encoding='utf-8') as f:
                        f.write(data)
                    playBackUrl = mpdFile
            li.setProperty(
                is_addon + '.license_key',
                self._licence_url.format(TOKEN=self.api.getToken(), licenseURL=licenseURL))
            li.setProperty(is_addon + '.manifest_type', protocol)
            li.setProperty('inputstream', is_addon)
            li.setPath(playBackUrl)
            # Start Playing
            xbmcplugin.setResolvedUrl(addon_handle, True, listitem=li)
        else:
            xbmcgui.Dialog().notification(
                'Abspielen fehlgeschlagen',
                'Es ist keine Abspiel URL vorhanden',
                icon=xbmcgui.NOTIFICATION_ERROR)

    def recommendationOverview(self):
        formats = self.api.getRecommendationOverview(cacheFile=self._recFile)
        for item in formats:
            url = build_url({'action': "listRecommendation", 'id': item.id})
            li = xbmcgui.ListItem(item.title)
            xbmcplugin.addDirectoryItem(
                handle=addon_handle, url=url, listitem=li, isFolder=True)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=True)

    def listRecommendation(self, recId):
        formats = self.api.getRecommendation(recId, self._recFile)
        for item in formats:
            action = "showMovie" if item.formatType == "Movie" else "listPage"
            url = build_url({'action': action, 'id': item.id})
            li = xbmcgui.ListItem(item.title)
            artDict = {}
            artDict['fanart'] = item.landscapeImg
            artDict['poster'] = item.portraitImage
            artDict['icon'] = item.portraitImage
            artDict['thumb'] = item.landscapeImg
            li.setArt(artDict)
            self._addContextMenuFav(item.id, li)
            xbmcplugin.addDirectoryItem(
                handle=addon_handle, url=url, listitem=li, isFolder=True)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=True)

    def listFavorites(self):
        formats = self.api.getWatchlist()
        favorites = []
        for item in formats:
            favorites.append(item.id)
            action = "showMovie" if item.formatType == "Movie" else "listPage"
            url = build_url({'action': action, 'id': item.id})
            li = xbmcgui.ListItem(item.title)
            artDict = {}
            artDict['fanart'] = item.landscapeImg
            artDict['poster'] = item.portraitImage
            artDict['icon'] = item.portraitImage
            artDict['thumb'] = item.landscapeImg
            li.setArt(artDict)
            self._addContextMenuFav(item.id, li)
            xbmcplugin.addDirectoryItem(
                handle=addon_handle, url=url, listitem=li, isFolder=True)
        self._db.set('favorites', favorites)
        xbmcplugin.endOfDirectory(addon_handle, cacheToDisc=True)

    def logout(self):
        self._addon.setSetting('acc_token', "")
        self._addon.setSetting('refresh_token',"")
        self._addon.setSetting('email', "")
        self._addon.setSetting('premium',"false")
        self._addon.setSetting('liveFree', "false")
        self._addon.setSetting('livePay', "false")

    def addFav(self, formatID):
        if self.api.modiyWishlist(formatID, 'addFavorite'):
            xbmcgui.Dialog().notification('Watchlist', 'Hinzufügen erfolgreich', icon=xbmcgui.NOTIFICATION_INFO)
        else:
            xbmcgui.Dialog().notification('Watchlist', 'Hinzufügen fehlgeschlagen', icon=xbmcgui.NOTIFICATION_ERROR)
        favs = self.api.getFavs()
        self._db.set('favorites', favs)


    def removeFav(self, formatID):
        if self.api.modiyWishlist(formatID, 'deleteFavorite'):
            xbmcgui.Dialog().notification('Watchlist', 'Entfernen erfolgreich', icon=xbmcgui.NOTIFICATION_INFO)
        else:
            xbmcgui.Dialog().notification('Watchlist', 'Entfernen fehlgeschlagen', icon=xbmcgui.NOTIFICATION_ERROR)
        favs = self.api.getFavs()
        self._db.set('favorites', favs)
